@@ -22,13 +22,14 @@ Z* -------------------------------------------------------------------
 
 #include"Base.h"
 #include"MemoryDebug.h"
-#include"OOMac.h"
 #include"Match.h"
 #include"Util.h"
 #include"Feedback.h"
 #include"Parse.h"
 #include"FileStream.h"
 
+#include "PyMOLGlobals.h"
+
 #ifndef int2
 typedef int int2[2];
 #endif
@@ -36,7 +37,7 @@ typedef int int2[2];
 CMatch *MatchNew(PyMOLGlobals * G, unsigned int na, unsigned int nb, int dist_mats)
 {
   unsigned int dim[2];
-  OOCalloc(G, CMatch);
+  auto I = new CMatch();
 
   I->na = na;
   I->nb = nb;
@@ -613,5 +614,5 @@ void MatchFree(CMatch * I)
   FreeP(I->mat);
   FreeP(I->smat);
   VLAFreeP(I->pair);
-  OOFreeP(I);
+  DeleteP(I);
 }
