@@ -20,7 +20,6 @@ Z* -------------------------------------------------------------------
 #include"Base.h"
 #include"Ray.h"
 #include"Setting.h"
-#include"os_gl.h"
 #include"os_gl_cgo.h"
 #include"Rep.h"
 #include"MemoryDebug.h"
@@ -259,6 +258,11 @@ inline uchar CLIP_NORMAL_VALUE(float cv){ return ((cv>1.f) ? 127 :
 
 #define CGO_CUSTOM_CYLINDER_ALPHA      0x41
 
+#define CGO_BEZIER 0x42
+#define CGO_BEZIER_SZ 12
+
+#define CGO_DRAW_BEZIER 0x43
+
 #define CGO_LIGHTING             0x0B50
 
 #define CGO_VERTEX_ARRAY         0x01
@@ -271,6 +275,13 @@ inline uchar CLIP_NORMAL_VALUE(float cv){ return ((cv>1.f) ? 127 :
 extern int CGO_sz[];
 size_t CGO_sz_size();
 
+// I think CGO rendering functions should not modify CGO's, so the
+// data pointer should be const. Current exception: `pickcolorsset`
+#define CGO_OP_DATA_CONST const
+typedef CGO_OP_DATA_CONST float* const* CGO_op_data;
+typedef void CGO_op(CCGORenderer * I, CGO_op_data);
+typedef CGO_op *CGO_op_fn;
+
 // CGOs are floating point arrays so we need to work with sizes in terms of floats
 template <typename T>
 static size_t fsizeof() { return sizeof(T) / sizeof(float); }
@@ -371,6 +382,23 @@ namespace cgo {
       int get_data_length() const { return nverts * 3 + n_data; };
     };
 
+    struct color {
+      static const int op_code = CGO_COLOR;
+      color(const float* _rgb) { copy3f(_rgb, rgb); }
+      float rgb[3]{};
+    };
+
+    struct sphere {
+      static const int op_code = CGO_SPHERE;
+      sphere(const float* _center, float _diameter)
+          : diamter(_diameter)
+      {
+        copy3f(_center, center);
+      }
+      float center[3]{};
+      float diamter{};
+    };
+
     struct sphere_buffers : op_with_data, op_with_draw_buffers {
       static const int op_code = CGO_DRAW_SPHERE_BUFFERS;
       sphere_buffers(int _num_spheres, int _ub_flags,
@@ -682,6 +710,50 @@ namespace cgo {
       cCylCap get_cap1() const { return static_cast<cCylCap>(int(cap1)); }
       cCylCap get_cap2() const { return static_cast<cCylCap>(int(cap2)); }
     };
+
+    struct cone {
+      static const int op_code = CGO_CONE;
+      cone(const float* _vertex1, const float* _vertex2, const float _radius1,
+          const float _radius2, const float* _color1, const float* _color2,
+          const float _cap1, const float _cap2)
+          : radius1(_radius1)
+          , radius2(_radius2)
+          , cap1(_cap1)
+          , cap2(_cap2)
+      {
+        copy3f(_vertex1, vertex1);
+        copy3f(_vertex2, vertex2);
+        copy3f(_color1, color1);
+        copy3f(_color2, color2);
+      };
+      float vertex1[3], vertex2[3], radius1, radius2, color1[3], color2[3];
+      float cap1, cap2;
+
+      cCylCap get_cap1() const { return static_cast<cCylCap>(int(cap1)); }
+      cCylCap get_cap2() const { return static_cast<cCylCap>(int(cap2)); }
+    };
+
+    struct bezier {
+      static const int op_code = CGO_BEZIER;
+      bezier(const float* _cpA, const float* _lhA, const float* _rhB,
+          const float* _cpB)
+      {
+        copy3f(_cpA, cpA);
+        copy3f(_lhA, lhA);
+        copy3f(_rhB, rhB);
+        copy3f(_cpB, cpB);
+      }
+      float cpA[3], lhA[3], rhB[3], cpB[3];
+    };
+
+    struct bezier_buffers : op_with_draw_buffers {
+      static const int op_code = CGO_DRAW_BEZIER;
+      bezier_buffers(size_t _vboid)
+          : vboid(_vboid)
+      {
+      }
+      std::size_t vboid{};
+    };
   };
 };
 
@@ -863,8 +935,6 @@ private:
   std::vector<std::unique_ptr<float[]>> _data_heap;
 };
 
-int CGORendererInit(PyMOLGlobals * G);
-void CGORendererFree(PyMOLGlobals * G);
 #define CGONew new CGO
 #define CGONewSized CGONew
 int CGOGetExtent(const CGO * I, float *mn, float *mx);
@@ -901,6 +971,13 @@ CGO* CGOOptimizeToVBONotIndexed(const CGO* I, int est = 0,
 CGO *CGOOptimizeSpheresToVBONonIndexed(const CGO * I, int est=0, bool addshaders=false, CGO *leftOverCGO=NULL);
 #define CGOOptimizeSpheresToVBONonIndexedNoShader(I, est) CGOOptimizeSpheresToVBONonIndexed(I, est, false, NULL)
 
+/**
+ * Creates a VBO-based Bezier CGO.
+ * @param cgo CGO that contains raw Bezier CGOs
+ * @return optimized (VBO-based) bezier CGOs
+ */
+CGO* CGOOptimizeBezier(const CGO* cgo);
+
 int CGOCheckComplex(CGO * I);
 int CGOPreloadFonts(CGO * I);
 
@@ -971,7 +1048,8 @@ int CGOWriteIndent(CGO * I, const char *str, float indent);
 int CGOBoundingBox(CGO *I, const float *min, const float *max);
 int CGOAccessibility(CGO * I, const float a);
 
-int CGODrawTexture(CGO *I, int texture_id, float *worldPos, float *screenMin, float *screenMax, float *textExtent);
+int CGODrawTexture(CGO* I, float* worldPos, float* screenMin, float* screenMax,
+    float* textExtent);
 int CGODrawConnector(CGO *I, float *targetPt3d, float *labelCenterPt3d, float text_width, float text_height, float *screenOffset, float *screenWorldOffset, float *connectorColor, short relativeMode, int draw_bkgrd, float bkgrd_transp, float *bkgrd_color, float rel_ext_length, float connectorWidth);
 CGO *CGOOptimizeLabels(const CGO * I, int est, bool addshaders=false);
 CGO *CGOOptimizeTextures(const CGO * I, int est);
@@ -1002,11 +1080,11 @@ int CGOAlphaTriangle(CGO * I,
 		     float a1, float a2, float a3, int reverse);
 void CGOSetZVector(CGO * I, float z0, float z1, float z2);
 struct GadgetSet;
-void CGORenderGLPicking(CGO * I, RenderInfo *info,
+void CGORenderPicking(CGO * I, RenderInfo *info,
                         PickContext * context, CSetting * set1, CSetting * set2, Rep *rep=NULL);
-void CGORenderGL(CGO * I, const float *color, CSetting * set1, CSetting * set2,
+void CGORender(CGO * I, const float *color, CSetting * set1, CSetting * set2,
                  RenderInfo * info, Rep *rep);
-void CGORenderGLAlpha(CGO * I, RenderInfo * info, bool calcDepth);
+void CGORenderAlpha(CGO * I, RenderInfo * info, bool calcDepth);
 int CGORenderRay(CGO * I, CRay * ray, RenderInfo * info, const float *color, ObjectGadgetRamp *ramp, CSetting * set1, CSetting * set2);
 void CGOReset(CGO * I);
 
@@ -1034,8 +1112,21 @@ bool CGOHasOperationsOfType(const CGO *I, int optype);
 bool CGOHasOperationsOfTypeN(const CGO *I, const std::set<int> &optype);
 bool CGOHasCylinderOperations(const CGO *I);
 bool CGOHasSphereOperations(const CGO *I);
+
+/**
+ * @param cgo input CGO
+ * @return true if CGO contains CGO_BEZIER operations
+ */
+bool CGOHasBezierOperations(const CGO* cgo);
 bool CGOFilterOutCylinderOperationsInto(const CGO *I, CGO *cgo);
 
+/**
+ * @param input input CGO
+ * @param[out] outputCGO filtered CGO
+ * @return true if CGO is filtered
+ */
+bool CGOFilterOutBezierOperationsInto(const CGO* inputCGO, CGO* outCGO);
+
 bool CGOCheckWhetherToFree(PyMOLGlobals * G, CGO *I);
 
 CGO *CGOConvertLinesToShaderCylinders(const CGO * I, int est);
@@ -1108,4 +1199,28 @@ CGO *CGOConvertShaderCylindersToCylinderShader(const CGO *I, CGO *addTo);
 void AssignNewPickColor(CGO* cgo, PickColorManager*, unsigned char* color,
     const PickContext* context, unsigned int index, int bond);
 
+constexpr int VerticesPerSphere() {
+#if defined(PURE_OPENGL_ES_2)
+  return 6;
+#else
+  return 4;
+#endif
+}
+
+constexpr int NumVerticesPerCylinder() {
+#if defined(_PYMOL_IOS) && !defined(_WEBGL)
+  return 4;
+#else
+  return 8;
+#endif
+}
+
+constexpr int NumTotalVerticesPerCylinder() {
+#if defined(_PYMOL_IOS) && !defined(_WEBGL)
+ return 6;
+#else
+ return 36;
+#endif
+}
+
 #endif
