@@ -21,9 +21,15 @@ Z* -------------------------------------------------------------------
 
 #include"Extrude.h"
 #include"Base.h"
-#include"OOMac.h"
+#include"Err.h"
 #include"Setting.h"
 #include"Feedback.h"
+#include "MemoryDebug.h"
+#include "CGO.h"
+#include "AtomInfo.h"
+#include "ObjectMolecule.h"
+
+#include "pymol/algorithm.h"
 
 static
 void ExtrudeInit(PyMOLGlobals * G, CExtrude * I);
@@ -33,7 +39,7 @@ void ExtrudeInit(PyMOLGlobals * G, CExtrude * I);
 CExtrude *ExtrudeCopyPointsNormalsColors(CExtrude * orig)
 {
   int ok = true;
-  OOAlloc(orig->G, CExtrude);
+  auto I = new CExtrude();
   CHECKOK(ok, I);
   if (ok)
     ExtrudeInit(orig->G, I);
@@ -477,11 +483,8 @@ int ExtrudeDumbbell2(CExtrude * I, int n, int sign, float length, float size)
 
 CExtrude *ExtrudeNew(PyMOLGlobals * G)
 {
-  int ok = true;
-  OOAlloc(G, CExtrude);
-  CHECKOK(ok, I);
-  if (ok)
-    ExtrudeInit(G, I);
+  auto I = new CExtrude();
+  ExtrudeInit(G, I);
   return (I);
 }
 
@@ -528,6 +531,135 @@ void ExtrudeBuildNormals2f(CExtrude * I)
 
 }
 
+/**
+ * Approximate a helix center trace.
+ *
+ * - shifts points to the helix center
+ * - smoothes the points curve (low-pass filter)
+ *
+ * @pre I has helix geometry
+ * @post I has linear geometry along helix axis
+ * @post I has normals
+ *
+ * @param[in,out] I data structure to modify in-place.
+ * @param radius Cylindrical helix radius (for optimizing end points)
+ * @param sampling Samples per residue
+ */
+void ExtrudeShiftToAxis(CExtrude* I, float radius, int sampling)
+{
+  assert(I->N > 1);
+
+  constexpr float loop_radius = 0.2f;
+  const int smooth_cycles =
+      SettingGet<int>(I->G, cSetting_cartoon_smooth_cylinder_cycles);
+  const int smooth_window =
+      SettingGet<int>(I->G, cSetting_cartoon_smooth_cylinder_window);
+
+  float p_start[3];
+  float p_end[3];
+
+  // original start and end points
+  copy3(I->p, p_start);
+  copy3(I->p + (I->N - 1) * 3, p_end);
+
+  ExtrudeBuildNormals2f(I);
+
+  // Because segments have open ends, we don't have ideal helix normals for the
+  // first and last position. We can reconstruct the desired normals from the
+  // next position by an ideal rotation.
+  if (I->N > 2) {
+#if 0
+    // dump residue rotations to figure out ideal rotation
+    for (int a = 1; a + 2 < I->N; ++a) {
+      const float* base0 = I->n + (a + 0) * 9;
+      const float* base1 = I->n + (a + 1) * 9;
+
+      float base0_inv[9];
+      float residue_rotation[9];
+
+      assert(fabs(determinant33f(base0) - 1.0) < 1e-3);
+      transpose33f33f(base0, base0_inv);
+      multiply33f33f(base1, base0_inv, residue_rotation);
+
+      printf("========= a %d\n", a);
+      dump33f(residue_rotation, "rotation");
+    }
+#endif
+
+    // ideal rotation from one alpha helix residue to the next
+    static float const residue_rotation[9] = {
+        0.224, -0.809, -0.544, 0.809, -0.157, 0.567, -0.544, -0.567, 0.619};
+    static float const residue_rotation_inv[9] = {
+        0.224, 0.809, -0.544, -0.809, -0.157, -0.567, -0.544, 0.567, 0.619};
+
+    multiply33f33f(residue_rotation_inv, //
+        I->n + 9 * sampling, I->n);
+    multiply33f33f(residue_rotation, //
+        I->n + 9 * (I->N - 1 - sampling), I->n + 9 * (I->N - 1));
+  }
+
+  // move points to helix axes
+  for (int a = 0; a < I->N; ++a) {
+    float* point = I->p + a * 3;
+    const float* normal = I->n + a * 9 + 3;
+
+    // distance to move point (distance between C-alpha atom and helix axis)
+    float factor = 2.3;
+
+    // keep end points close enough to adjacent segments so they overlap
+    if (a == 0 || a + 1 == I->N) {
+      factor = std::min(factor, radius - loop_radius);
+    }
+
+    float tmp[3];
+    scale3f(normal, -factor, tmp);
+    add3f(point, tmp, point);
+  }
+
+  // window averaging of point positions
+  if (I->N > 2 && smooth_window > 0) {
+    int const w2 = sampling * smooth_window;
+
+    for (int i = 0; i < smooth_cycles; ++i) {
+      std::vector<float> smoothed((I->N - 2) * 3);
+
+      for (int a = 1; a + 1 < I->N; ++a) {
+        float* avg = smoothed.data() + (a - 1) * 3;
+
+        for (int j = -w2; j <= w2; ++j) {
+          int const k = pymol::clamp(a + j, 0, I->N - 1);
+          add3f(I->p + k * 3, avg, avg);
+        }
+
+        scale3f(avg, 1. / (2 * w2 + 1), avg);
+      }
+
+      std::copy(smoothed.begin(), smoothed.end(), I->p + 3);
+    }
+  }
+
+  ExtrudeComputeTangents(I);
+  ExtrudeBuildNormals1f(I);
+
+  // extend tips for better geometry overlap with adjacent segments
+  auto const push_out_tip = [&](int index, float const* pos_orig,
+                                int direction) {
+    float const protrusion = loop_radius * 2;
+    float const* normal = I->n + index * 9;
+    float* pos = I->p + index * 3;
+    float offset[3];
+    subtract3f(pos_orig, pos, offset);
+    float const len = project3f(offset, normal, offset) * direction;
+    if (len > -protrusion) {
+      scale3f(normal, (len + protrusion) * direction, offset);
+      add3f(pos, offset, pos);
+    }
+  };
+
+  push_out_tip(0, p_start, -1);
+  push_out_tip(I->N - 1, p_end, 1);
+}
+
 #if 0
 /* Is this ever used? */
 void ExtrudeCGOTraceAxes(CExtrude * I, CGO * cgo)
@@ -678,7 +810,7 @@ void ExtrudeCGOTraceFrame(CExtrude * I, CGO * cgo)
 }
 #endif
 
-/*
+/**
  * Draw flat cap on a tube cartoon (loop, oval, etc.)
  *
  * I: tube instance
@@ -723,7 +855,7 @@ void TubeCapFlat(const CExtrude * I, CGO * cgo, int index, bool inv_dir, const f
   CGOPickColor(cgo, -1, cPickableNoPick);
 }
 
-/*
+/**
  * I: tube instance
  * cgo: CGO to add to
  * cap: 0: no caps, 1: flat caps, 2: round caps
@@ -731,7 +863,8 @@ void TubeCapFlat(const CExtrude * I, CGO * cgo, int index, bool inv_dir, const f
  * use_spheres: do round caps with spheres instead of triangles
  * dash: if > 0, skip every segment which is a multiple of `dash`
  */
-int ExtrudeCGOSurfaceTube(CExtrude * I, CGO * cgo, int cap, const float *color_override, bool use_spheres, int dash)
+int ExtrudeCGOSurfaceTube(const CExtrude* I, CGO* cgo, cCylCap cap,
+    const float* color_override, bool use_spheres, int dash)
 {
   int a, b;
   unsigned int *i;
@@ -845,7 +978,7 @@ int ExtrudeCGOSurfaceTube(CExtrude * I, CGO * cgo, int cap, const float *color_o
 	  ok &= CGOPickColor(cgo, -1, cPickableNoPick);
       }
 
-      if (cap == 1) {
+      if (cap == cCylCap::Flat) {
         TubeCapFlat(I, cgo, a_start, true, color_override);
         TubeCapFlat(I, cgo, a_end - 1, false, color_override);
       }
@@ -853,7 +986,7 @@ int ExtrudeCGOSurfaceTube(CExtrude * I, CGO * cgo, int cap, const float *color_o
 
     if (ok){
     switch (cap) {
-    case 2:
+    case cCylCap::Round:
       {
 	float p0[3], p1[3], p2[3], z1, z2, normal[3], vertex1[3];
 	float c, d, prev, x, y, *v1, nEdge = I->Ns, nEdgeH = 2.f * floor(I->Ns/2.f);
@@ -1098,7 +1231,7 @@ int ExtrudeCylindersToCGO(CExtrude * I, CGO * cgo, float tube_radius){
   return ok;
 }
 
-int ExtrudeCGOSurfaceVariableTube(CExtrude * I, CGO * cgo, int cap)
+int ExtrudeCGOSurfaceVariableTube(const CExtrude* I, CGO* cgo, cCylCap cap)
 {
   int a, b;
   unsigned int *i;
@@ -1295,7 +1428,7 @@ int ExtrudeCGOSurfaceVariableTube(CExtrude * I, CGO * cgo, int cap)
       }
     }
 
-    if(ok && cap) {
+    if(ok && cap != cCylCap::None) {
 
       n = I->n;
       v = I->p;
@@ -1372,7 +1505,7 @@ int ExtrudeCGOSurfaceVariableTube(CExtrude * I, CGO * cgo, int cap)
   return ok;
 }
 
-int ExtrudeCGOSurfacePolygon(CExtrude * I, CGO * cgo, int cap, const float *color_override)
+int ExtrudeCGOSurfacePolygon(const CExtrude * I, CGO * cgo, cCylCap cap, const float *color_override)
 {
   int a, b;
   unsigned int *i;
@@ -1476,7 +1609,7 @@ int ExtrudeCGOSurfacePolygon(CExtrude * I, CGO * cgo, int cap, const float *colo
 	ok &= CGOPickColor(cgo, -1, cPickableNoPick);
     }
 
-    if(ok && cap) {
+    if(ok && cap != cCylCap::None) {
 
       if(color_override)
         ok &= CGOColorv(cgo, color_override);
@@ -1567,7 +1700,7 @@ int ExtrudeCGOSurfacePolygon(CExtrude * I, CGO * cgo, int cap, const float *colo
   return ok;
 }
 
-int ExtrudeCGOSurfacePolygonTaper(CExtrude * I, CGO * cgo, int sampling,
+int ExtrudeCGOSurfacePolygonTaper(const CExtrude * I, CGO * cgo, int sampling,
 				  const float *color_override)
 {
   int a, b;
@@ -1710,7 +1843,7 @@ int ExtrudeCGOSurfacePolygonTaper(CExtrude * I, CGO * cgo, int sampling,
   return ok;
 }
 
-int ExtrudeCGOSurfaceStrand(CExtrude * I, CGO * cgo, int sampling, const float *color_override)
+int ExtrudeCGOSurfaceStrand(const CExtrude * I, CGO * cgo, int sampling, const float *color_override)
 {
   int a, b;
   unsigned int *i;
@@ -2162,7 +2295,7 @@ int ExtrudeComputePuttyScaleFactors(CExtrude * I, ObjectMolecule * obj, int tran
       PRINTFB(I->G, FB_RepCartoon, FB_Warnings)
         " Extrude-Warning: invalid putty settings (division by zero)\n" ENDFB(I->G);
       for(a = 0; a < I->N; a++) {
-        *sf = 0.0F;
+        *sf = 0.5F;
         sf++;
       }
     }
@@ -2271,5 +2404,5 @@ void ExtrudeFree(CExtrude * I)
   FreeP(I->sv);
   FreeP(I->i);
   FreeP(I->sf);
-  OOFreeP(I);
+  DeleteP(I);
 }
