@@ -33,6 +33,7 @@ Z* -------------------------------------------------------------------
 #include"Movie.h"
 #include"View.h"
 #include"Err.h"
+#include "Feedback.h"
 
 #include"Executive.h"
 #include"CGO.h"
@@ -105,99 +106,6 @@ int ObjectMotionModify(pymol::CObject *I,int action, int index, int count,int ta
   return ok;
 }
 
-static void TTTToViewElem(float *TTT, CViewElem * elem)
-{
-  float *fp = TTT;
-  double *dp;
-
-  /* convert row-major TTT to column-major ViewElem */
-
-  elem->matrix_flag = true;
-  dp = elem->matrix;
-
-  dp[0] = (double) fp[0];
-  dp[1] = (double) fp[4];
-  dp[2] = (double) fp[8];
-  dp[3] = 0.0;
-
-  dp[4] = (double) fp[1];
-  dp[5] = (double) fp[5];
-  dp[6] = (double) fp[9];
-  dp[7] = 0.0;
-
-  dp[8] = (double) fp[2];
-  dp[9] = (double) fp[6];
-  dp[10] = (double) fp[10];
-  dp[11] = 0.0;
-
-  dp[12] = 0.0;
-  dp[13] = 0.0;
-  dp[14] = 0.0;
-  dp[15] = 1.0;
-
-  /* copy inverse pre */
-
-  elem->pre_flag = true;
-  dp = elem->pre;
-  *(dp++) = (double) -TTT[12];
-  *(dp++) = (double) -TTT[13];
-  *(dp++) = (double) -TTT[14];
-
-  /* copy post */
-
-  elem->post_flag = true;
-  dp = elem->post;
-  *(dp++) = (double) TTT[3];
-  *(dp++) = (double) TTT[7];
-  *(dp++) = (double) TTT[11];
-
-}
-
-static void TTTFromViewElem(float *TTT, CViewElem * elem)
-{
-  float *fp = TTT;
-  double *dp;
-
-  if(elem->matrix_flag) {
-    dp = elem->matrix;
-
-    fp[0] = (float) dp[0];
-    fp[1] = (float) dp[4];
-    fp[2] = (float) dp[8];
-    fp[3] = 0.0;
-
-    fp[4] = (float) dp[1];
-    fp[5] = (float) dp[5];
-    fp[6] = (float) dp[9];
-    fp[7] = 0.0;
-
-    fp[8] = (float) dp[2];
-    fp[9] = (float) dp[6];
-    fp[10] = (float) dp[10];
-    fp[11] = 0.0;
-
-    fp[12] = 0.0;
-    fp[13] = 0.0;
-    fp[14] = 0.0;
-    fp[15] = 1.0;
-  }
-
-  if(elem->pre_flag) {
-    dp = elem->pre;
-    fp[12] = (float) (-*(dp++));
-    fp[13] = (float) (-*(dp++));
-    fp[14] = (float) (-*(dp++));
-  }
-
-  if(elem->post_flag) {
-    dp = elem->post;
-    fp[3] = (float) *(dp++);
-    fp[7] = (float) *(dp++);
-    fp[11] = (float) *(dp++);
-  }
-  fp[15] = 1.0F;
-}
-
 int ObjectGetSpecLevel(pymol::CObject * I, int frame)
 {
   if(I->ViewElem) {
@@ -218,10 +126,10 @@ int ObjectGetSpecLevel(pymol::CObject * I, int frame)
   return -1;
 }
 
-void ObjectDrawViewElem(pymol::CObject *I, BlockRect *rect,int frames ORTHOCGOARG)
+void ObjectDrawViewElem(pymol::CObject *I, BlockRect *rect,int frames , CGO *orthoCGO)
 {
   if(I->ViewElem) {
-    ViewElemDraw(I->G,I->ViewElem,rect,frames,I->Name ORTHOCGOARGVAR);
+    ViewElemDraw(I->G,I->ViewElem,rect,frames,I->Name, orthoCGO);
   }
 }
 
@@ -770,7 +678,7 @@ int ObjectGetCurrentState(const pymol::CObject * I, int ignore_all_states)
   return std::max(-1, I->getCurrentState());
 }
 
-PyObject *ObjectAsPyList(pymol::CObject * I)
+PyObject *ObjectAsPyList(const pymol::CObject * I)
 {
   PyObject *result = NULL;
   result = PyList_New(14);
@@ -1227,6 +1135,10 @@ void ObjectSetName(pymol::CObject * I, const char *name)
     ObjectMakeValidName(I->G, I->Name);
 }
 
+void pymol::CObject::setName(pymol::zstring_view name)
+{
+  ObjectSetName(this, name.data());
+}
 
 /*========================================================================*/
 pymol::CObject::~CObject()
