@@ -19,11 +19,12 @@ Z* -------------------------------------------------------------------
 
 #include <vector>
 #include <string>
+#include <unordered_map>
 
 #include"os_python.h"
 #include"PyMOLGlobals.h"
-#include"OVOneToOne.h"
 #include"pymol/memory.h"
+#include"Result.h"
 
 typedef char SettingName[255];
 
@@ -41,10 +42,11 @@ struct SettingUniqueEntry {
 };
 
 struct CSettingUnique {
-  OVOneToOne *id2offset;
-  OVOneToOne *old2new;
-  SettingUniqueEntry *entry;
-  int n_alloc, next_free;
+  std::unordered_map<int, int> id2offset;
+  std::unique_ptr<std::unordered_map<int, int>> old2new;
+  std::vector<SettingUniqueEntry> entry;
+  constexpr static int numInitEntries = 10;
+  int next_free{};
 };
 
 /**
@@ -293,6 +295,29 @@ public:
   };
 };
 
+/**
+ * Similar to StateIterator but easily useable
+ * with ranged-for loops
+ */
+class StateIteratorV2 {
+  StateIndex_t m_beg;
+  StateIndex_t m_end;
+  StateIndex_t m_cur;
+
+public:
+  StateIteratorV2(pymol::CObject*, StateIndex_t);
+
+  /**
+   * @return begin state iterator
+   */
+  StateIndex_t* begin();
+
+  /**
+   * @return end state iterator
+   */
+  StateIndex_t* end();
+};
+
 /*
  * Setting levels (global, object, ...)
  */
@@ -320,8 +345,8 @@ const char * SettingLevelGetName(unsigned index);
 bool SettingLevelCheckMask(PyMOLGlobals * G, int index, unsigned char mask);
 bool SettingLevelCheck(PyMOLGlobals * G, int index, unsigned char level);
 
-bool CPyMOLInitSetting(OVLexicon * Lex, OVOneToOne * Setting);
-extern "C" OVreturn_word get_setting_id(CPyMOL * I, const char *setting);
+bool CPyMOLInitSetting(OVLexicon * Lex, std::unordered_map<int, int>& Setting);
+pymol::Result<int> get_setting_id(CPyMOL* I, const char* setting);
 
 /*
  * Overloaded setters for templatted programming
