@@ -25,6 +25,12 @@ Z* -------------------------------------------------------------------
 #include"ObjectMolecule.h"
 #include"vla.h"
 
+#include "pymol/math_defines.h"
+#include "pymol/memory.h"
+#include "pymol/zstring_view.h"
+
+#include <type_traits>
+
 #define COORD_SET_HAS_ANISOU 0x01
 
 enum mmpymolx_prop_state_t {
@@ -40,17 +46,22 @@ struct CoordSet : CObjectState {
     Octahedral = 2,
   };
 
-  // methods (not fully refactored yet)
-  void fFree();
-
   // methods
   void update(int state);
   void render(RenderInfo * info);
   void enumIndices();
-  void appendIndices(int offset);
   int extendIndices(int nAtom);
-  void invalidateRep(int type, int level);
+  void invalidateRep(cRep_t type, cRepInv_t level);
   int atmToIdx(int atm) const;
+  void setNIndex(unsigned nindex);
+  void updateNonDiscreteAtmToIdx(unsigned);
+
+  /// Number of atoms with coordinates in this coordset
+  int getNIndex() const
+  {
+    // TODO If possible, eliminate NIndex, use IdxToAtm.size()
+    return NIndex;
+  }
 
   // read/write pointer to coordinate
   float * coordPtr(int idx) {
@@ -62,10 +73,17 @@ struct CoordSet : CObjectState {
     return Coord + idx * 3;
   }
 
+  float const* coordPtrSym(
+      int idx, pymol::SymOp const& symop, float* v_out, bool inv = false) const;
+
   AtomInfoType * getAtomInfo(int idx) {
     return Obj->AtomInfo + IdxToAtm[idx];
   }
 
+  const AtomInfoType* getAtomInfo(int idx) const {
+    return const_cast<CoordSet*>(this)->getAtomInfo(idx);
+  }
+
   // true if any atom in this coord set has any of the reps in "bitmask" shown
   bool hasRep(int bitmask) {
     if (Obj->RepVisCache & bitmask)
@@ -75,11 +93,17 @@ struct CoordSet : CObjectState {
     return false;
   }
 
+  //! Get symmetry if defined, otherwise get it from parent object.
+  CSymmetry const* getSymmetry() const
+  {
+    return Symmetry ? Symmetry.get() : Obj ? Obj->Symmetry.get() : nullptr;
+  }
+
   ObjectMolecule *Obj = nullptr;
   pymol::vla<float> Coord;
-  pymol::vla<int> IdxToAtm;
-  pymol::vla<int> AtmToIdx;
-  int NIndex = 0, NAtIndex = 0, prevNIndex = 0, prevNAtIndex = 0;
+  std::vector<int> IdxToAtm;
+  std::vector<int> AtmToIdx;
+  int NIndex = 0;
   ::Rep *Rep[cRepCnt] = {0};            /* an array of pointers to representations */
   int Active[cRepCnt] = {0};          /* active flags */
   int NTmpBond = 0;                 /* optional, temporary (for coord set transfers) */
@@ -90,14 +114,10 @@ struct CoordSet : CObjectState {
   WordType Name = {0};
   std::vector<float> Spheroid;
   std::vector<float> SpheroidNormal;
-  CSetting *Setting = nullptr;
-  /* for periodic MD boxes -- may be merge into symmetry lattice later... */
-  std::unique_ptr<CCrystal> PeriodicBox;
+  pymol::copyable_ptr<CSetting> Setting;
   int PeriodicBoxType = NoPeriodicity;
   int tmp_index = 0;                /* for saving */
 
-  pymol::vla<LabPosType> LabPos;
-
   /* not saved in state */
 
   pymol::vla<RefPosType> RefPos;
@@ -113,6 +133,9 @@ struct CoordSet : CObjectState {
 
   CGO *SculptCGO = nullptr;
   CGO *SculptShaderCGO = nullptr;
+  pymol::cache_ptr<CGO> UnitCellCGO;
+  pymol::cache_ptr<CGO> UnitCellShaderCGO;
+
   MapType *Coord2Idx = nullptr;
   float Coord2IdxReq = 0, Coord2IdxDiv = 0;
 
@@ -129,18 +152,31 @@ struct CoordSet : CObjectState {
 
   /* Atom-state Settings */
   pymol::vla<int> atom_state_setting_id;
-  pymol::vla<char> has_atom_state_settings;
+
+  /// True if any atom might have state level settings
+  bool has_any_atom_state_settings() const { return atom_state_setting_id; }
+
+  /// True if atom `idx` has state level settings
+  bool has_atom_state_settings(int idx) const
+  {
+    return atom_state_setting_id && atom_state_setting_id[idx] != 0;
+  }
 
   // special member functions
   CoordSet(PyMOLGlobals * G);
   CoordSet(const CoordSet &cs);
   ~CoordSet();
+
+  pymol::Result<pymol::Vec3> getAtomLabelOffset(int atm) const;
+  pymol::Result<> setAtomLabelOffset(int atm, const float* offset);
+
+  void setTitle(pymol::zstring_view);
+
+  double const* getPremultipliedMatrix() const;
 };
 
-typedef void (*fUpdateFn) (CoordSet *, int);
-
-int BondInOrder(BondType * a, int b1, int b2);
-int BondCompare(BondType * a, BondType * b);
+int BondInOrder(BondType const* a, int b1, int b2);
+int BondCompare(BondType const* a, BondType const* b);
 
 PyObject *CoordSetAsNumPyArray(CoordSet * cs, short copy);
 PyObject *CoordSetAsPyList(CoordSet * I);
@@ -179,12 +215,14 @@ int CoordSetTransformAtomR44f(CoordSet * I, int at, const float *matrix);
 
 int CoordSetValidateRefPos(CoordSet * I);
 
-void CoordSetPurge(CoordSet * I);
-void CoordSetAdjustAtmIdx(CoordSet * I, int *lookup, int nAtom);
-int CoordSetMerge(ObjectMolecule *OM, CoordSet * I, CoordSet * cs);        /* must be non-overlapping */
+void CoordSetAdjustAtmIdx(CoordSet*, const int*);
+int CoordSetMerge(ObjectMolecule *OM, CoordSet * I, const CoordSet * cs);        /* must be non-overlapping */
 void CoordSetRecordTxfApplied(CoordSet * I, const float *TTT, int homogenous);
 void CoordSetUpdateCoord2IdxMap(CoordSet * I, float cutoff);
 
+bool CoordSetFindOpenValenceVector(const CoordSet*, int atm, float* out,
+    const float* seek = nullptr, int ignore_atm = -1);
+
 typedef struct _CCoordSetUpdateThreadInfo CCoordSetUpdateThreadInfo;
 
 void CoordSetUpdateThread(CCoordSetUpdateThreadInfo * T);
@@ -218,13 +256,21 @@ template <typename V> void AtomStateGetSetting(ATOMSTATEGETSETTINGARGS, V * out)
 template <typename V> void SettingSet(int index, V value, CoordSet *cs, int idx) {
   auto& G = cs->G;
   CoordSetCheckUniqueID(G, cs, idx);
-  cs->has_atom_state_settings[idx] = true;
   SettingUniqueSet(G, cs->atom_state_setting_id[idx], index, value);
 }
 
 // object-state level setting
 template <typename V> void SettingSet(int index, V value, CoordSet *cs) {
-  SettingSet(cs->G, &cs->Setting, index, value);
+  SettingSet(cs->G, cs->Setting, index, value);
+}
+
+//! Get object-state level setting
+template <typename V>
+V SettingGet(const CoordSet& cs, int index)
+{
+  using T = typename std::conditional<std::is_enum<V>::value, int, V>::type;
+  return static_cast<V>(
+      SettingGet<T>(cs.G, cs.Setting.get(), cs.Obj->Setting.get(), index));
 }
 
 // Rotates the ANISOU vector
