@@ -15,7 +15,7 @@ I* Additional authors of this source file include:
 Z* -------------------------------------------------------------------
 */
 
-#include <set>
+#include <unordered_set>
 
 #include"os_python.h"
 
@@ -23,7 +23,6 @@ Z* -------------------------------------------------------------------
 #include"os_std.h"
 #include"os_gl.h"
 
-#include"OOMac.h"
 #include"ObjectAlignment.h"
 #include"Base.h"
 #include"MemoryDebug.h"
@@ -34,7 +33,6 @@ Z* -------------------------------------------------------------------
 #include"main.h"
 #include"Color.h"
 #include"Executive.h"
-#include"OVContext.h"
 #include"Util.h"
 #include"Selector.h"
 #include"Seq.h"
@@ -484,7 +482,7 @@ static int *AlignmentMerge(PyMOLGlobals * G, int *curVLA, const int *newVLA,
 
       // get the set of non-guide objects in the new alignment, they need
       // to be flushed (removed) from the current alignment
-      std::set<const ObjectMolecule*> flushobjects;
+      std::unordered_set<const ObjectMolecule*> flushobjects;
       if (flush) {
         flushobjects.insert(flush);
       } else {
@@ -566,8 +564,8 @@ static int *AlignmentMerge(PyMOLGlobals * G, int *curVLA, const int *newVLA,
       /* now combine the alignments */
 
       {
-        OVOneToAny *used = OVOneToAny_New(G->Context->heap);
-        OVOneToAny *active = OVOneToAny_New(G->Context->heap);
+        std::unordered_set<int> used;
+        std::unordered_set<int> active;
         int cur_start = 0;
         int new_start = 0;
 
@@ -597,15 +595,15 @@ static int *AlignmentMerge(PyMOLGlobals * G, int *curVLA, const int *newVLA,
             int c, id;
             int overlapping = false;
 
-            OVOneToAny_Reset(active);
+            active.clear();
             c = cur_start;
             while((id = curVLA[c++])) { /* record active atoms */
-              OVOneToAny_SetKey(active, id, 1);
+              active.insert(id);
             }
 
             c = new_start;
             while((id = newVLA[c++])) { /* see if there are any matches */
-              if(OVreturn_IS_OK(OVOneToAny_GetKey(active, id))) {
+              if (active.find(id) != active.end()) {
                 overlapping = true;
                 break;
               }
@@ -658,12 +656,11 @@ static int *AlignmentMerge(PyMOLGlobals * G, int *curVLA, const int *newVLA,
             case -1:           /* insert new */
               if(new_start < n_new) {
                 while((id = newVLA[new_start])) {
-                  if(OVOneToAny_GetKey(used, id).status == OVstatus_NOT_FOUND) {
-                    if(OVreturn_IS_OK(OVOneToAny_SetKey(used, id, 1))) {
-                      VLACheck(result, int, n_result);
-                      result[n_result] = id;
-                      n_result++;
-                    }
+                  if (used.find(id) == used.end()) {
+                    used.insert(id);
+                    VLACheck(result, int, n_result);
+                    result[n_result] = id;
+                    n_result++;
                   }
                   new_start++;
                 }
@@ -677,12 +674,11 @@ static int *AlignmentMerge(PyMOLGlobals * G, int *curVLA, const int *newVLA,
             case 0:            /* merge, with cur going first */
               if(new_start < n_new) {
                 while((id = newVLA[new_start])) {
-                  if(OVOneToAny_GetKey(used, id).status == OVstatus_NOT_FOUND) {
-                    if(OVreturn_IS_OK(OVOneToAny_SetKey(used, id, 1))) {
-                      VLACheck(result, int, n_result);
-                      result[n_result] = id;
-                      n_result++;
-                    }
+                  if (used.find(id) == used.end()) {
+                    used.insert(id);
+                    VLACheck(result, int, n_result);
+                    result[n_result] = id;
+                    n_result++;
                   }
                   new_start++;
                 }
@@ -691,12 +687,11 @@ static int *AlignmentMerge(PyMOLGlobals * G, int *curVLA, const int *newVLA,
               }
               if(cur_start < n_cur) {
                 while((id = curVLA[cur_start])) {
-                  if(OVOneToAny_GetKey(used, id).status == OVstatus_NOT_FOUND) {
-                    if(OVreturn_IS_OK(OVOneToAny_SetKey(used, id, 1))) {
-                      VLACheck(result, int, n_result);
-                      result[n_result] = id;
-                      n_result++;
-                    }
+                  if (used.find(id) == used.end()) {
+                    used.insert(id);
+                    VLACheck(result, int, n_result);
+                    result[n_result] = id;
+                    n_result++;
                   }
                   cur_start++;
                 }
@@ -710,12 +705,11 @@ static int *AlignmentMerge(PyMOLGlobals * G, int *curVLA, const int *newVLA,
             case 1:            /* insert cur */
               if(cur_start < n_cur) {
                 while((id = curVLA[cur_start])) {
-                  if(OVOneToAny_GetKey(used, id).status == OVstatus_NOT_FOUND) {
-                    if(OVreturn_IS_OK(OVOneToAny_SetKey(used, id, 1))) {
-                      VLACheck(result, int, n_result);
-                      result[n_result] = id;
-                      n_result++;
-                    }
+                  if (used.find(id) == used.end()) {
+                    used.insert(id);
+                    VLACheck(result, int, n_result);
+                    result[n_result] = id;
+                    n_result++;
                   }
                   cur_start++;
                 }
@@ -729,8 +723,6 @@ static int *AlignmentMerge(PyMOLGlobals * G, int *curVLA, const int *newVLA,
             }
           }
         }
-        OVOneToAny_DEL_AUTO_NULL(active);
-        OVOneToAny_DEL_AUTO_NULL(used);
       }
     }
   }
@@ -1142,7 +1134,7 @@ void ObjectAlignment::render(RenderInfo * info)
           }
 
           if (cgo) {
-            CGORenderGL(cgo, color, I->Setting.get(), NULL, info, NULL);
+            CGORender(cgo, color, I->Setting.get(), NULL, info, NULL);
           }
 
 #ifndef PURE_OPENGL_ES_2
