@@ -20,7 +20,6 @@ Z* -------------------------------------------------------------------
 #include"os_std.h"
 #include"os_gl.h"
 
-#include"OOMac.h"
 #include"ObjectGadgetRamp.h"
 #include"GadgetSet.h"
 #include"Base.h"
@@ -37,6 +36,7 @@ Z* -------------------------------------------------------------------
 #include"Util.h"
 #include"P.h"
 #include"PyMOLObject.h"
+#include "Feedback.h"
 
 static void ObjectGadgetRampBuild(ObjectGadgetRamp *);
 static int ObjectGadgetRampHandleInputColors(ObjectGadgetRamp *);
