@@ -20,7 +20,7 @@ Z* -------------------------------------------------------------------
 #include"os_std.h"
 #include"os_gl.h"
 
-#include"OOMac.h"
+#include"Err.h"
 #include"ObjectMesh.h"
 #include"Base.h"
 #include"MemoryDebug.h"
@@ -38,6 +38,7 @@ Z* -------------------------------------------------------------------
 #include"Matrix.h"
 #include"ShaderMgr.h"
 #include"ObjectCGO.h"
+#include "Feedback.h"
 
 static void ObjectMeshRecomputeExtent(ObjectMesh * I);
 
@@ -721,7 +722,7 @@ static short ObjectMeshStateRenderShader(ObjectMeshState *ms, ObjectMesh *I,
 		     SceneGetTwoSidedLighting(G));
   }
 
-  CGORenderGL(ms->shaderCGO.get(), NULL, NULL, NULL, info, NULL);
+  CGORender(ms->shaderCGO.get(), NULL, NULL, NULL, info, NULL);
 
   if (shaderPrg) {
     shaderPrg->Disable();
@@ -730,7 +731,7 @@ static short ObjectMeshStateRenderShader(ObjectMeshState *ms, ObjectMesh *I,
   if (ms->shaderUnitCellCGO){
     shaderPrg = G->ShaderMgr->Enable_DefaultShader(info->pass);
     shaderPrg->SetLightingEnabled(0);
-    CGORenderGL(ms->shaderUnitCellCGO.get(), NULL, NULL, NULL, info, NULL);
+    CGORender(ms->shaderUnitCellCGO.get(), NULL, NULL, NULL, info, NULL);
     shaderPrg->Disable();
   }
 
@@ -897,7 +898,7 @@ static CGO *ObjectMeshRenderImpl(ObjectMesh * I, RenderInfo * info, int returnCG
 	      if(ms->UnitCellCGO && (I->visRep & cRepCellBit)) {
 		const float *color = ColorGet(I->G, I->Color);
 		if (!use_shader) {
-		  CGORenderGL(ms->UnitCellCGO.get(), color, I->Setting.get(), NULL, info, NULL);
+		  CGORender(ms->UnitCellCGO.get(), color, I->Setting.get(), NULL, info, NULL);
 		} else if(!ms->shaderUnitCellCGO) {
 		  CGO *newUnitCellCGO = CGONewSized(G, 0);
 		  CGOColorv(newUnitCellCGO, color);
