@@ -22,7 +22,7 @@ Z* -------------------------------------------------------------------
 
 #include"Base.h"
 #include"MemoryDebug.h"
-#include"OOMac.h"
+#include"Err.h"
 #include"RepMesh.h"
 #include"Map.h"
 #include"Isosurf.h"
@@ -34,6 +34,7 @@ Z* -------------------------------------------------------------------
 #include"PyMOLGlobals.h"
 #include"Selector.h"
 #include "ShaderMgr.h"
+#include "Feedback.h"
 
 struct RepMesh : Rep {
   using Rep::Rep;
@@ -409,7 +410,7 @@ void RepMesh::render(RenderInfo* info)
 	    I->shaderCGO->use_shader = true;
 	  generate_shader_cgo = 1;
 	} else if (ok) {
-	  CGORenderGL(I->shaderCGO, NULL, NULL, NULL, info, I);
+	  CGORender(I->shaderCGO, NULL, NULL, NULL, info, I);
 	  return;
 	}
       }
@@ -526,7 +527,7 @@ void RepMesh::render(RenderInfo* info)
 	  {
 	    const float *color;
 	    color = ColorGet(G, I->obj->Color);
-	    CGORenderGL(I->shaderCGO, color, NULL, NULL, info, I);
+	    CGORender(I->shaderCGO, color, NULL, NULL, info, I);
 	  }
 	}
       }
