@@ -21,7 +21,7 @@ Z* -------------------------------------------------------------------
 #include"os_gl.h"
 
 #include"Base.h"
-#include"OOMac.h"
+#include"Err.h"
 #include"RepNonbondedSphere.h"
 #include"Color.h"
 #include"Sphere.h"
@@ -30,45 +30,43 @@ Z* -------------------------------------------------------------------
 #include"ShaderMgr.h"
 #include"Scene.h"
 #include"CGO.h"
+#include "CoordSet.h"
+
+struct RepNonbondedSphere : Rep {
+  using Rep::Rep;
+
+  ~RepNonbondedSphere() override;
+
+  cRep_t type() const override { return cRepNonbondedSphere; }
+  void render(RenderInfo* info) override;
 
-typedef struct RepNonbondedSphere {
-  Rep R;
   CGO *shaderCGO, *primitiveCGO;
-} RepNonbondedSphere;
+};
 
 #include"ObjectMolecule.h"
 
-static
-void RepNonbondedSphereFree(RepNonbondedSphere * I)
+RepNonbondedSphere::~RepNonbondedSphere()
 {
-  if (I->shaderCGO){
-    CGOFree(I->shaderCGO);
-    I->shaderCGO = 0;
-  }
-  if (I->primitiveCGO){
-    CGOFree(I->primitiveCGO);
-    I->primitiveCGO = 0;
-  }
-  RepPurge(&I->R);
-  OOFreeP(I);
+  CGOFree(shaderCGO);
+  CGOFree(primitiveCGO);
 }
 
-static void RepNonbondedSphereRender(RepNonbondedSphere * I, RenderInfo * info)
+void RepNonbondedSphere::render(RenderInfo* info)
 {
+  auto I = this;
   CRay *ray = info->ray;
   auto pick = info->pick;
-  PyMOLGlobals *G = I->R.G;
 
   if(ray) {
 #ifndef _PYMOL_NO_RAY
-    CGORenderRay(I->primitiveCGO, ray, info, NULL, NULL, I->R.cs->Setting, I->R.obj->Setting);
+    CGORenderRay(I->primitiveCGO, ray, info, NULL, NULL, I->cs->Setting.get(), I->obj->Setting.get());
 #endif
   } else if(G->HaveGUI && G->ValidContext) {
     if(pick) {
       if (I->shaderCGO){
-	CGORenderGLPicking(I->shaderCGO, info, &I->R.context, I->R.cs->Setting, I->R.obj->Setting);
+	CGORenderPicking(I->shaderCGO, info, &I->context, I->cs->Setting.get(), I->obj->Setting.get());
       } else if (I->primitiveCGO){
-	CGORenderGLPicking(I->primitiveCGO, info, &I->R.context, I->R.cs->Setting, I->R.obj->Setting);
+	CGORenderPicking(I->primitiveCGO, info, &I->context, I->cs->Setting.get(), I->obj->Setting.get());
       }
     } else { /* rendering */
       short use_shader, use_sphere_shader;
@@ -88,23 +86,22 @@ static void RepNonbondedSphereRender(RepNonbondedSphere * I, RenderInfo * info)
           if (use_sphere_shader){
             I->shaderCGO = CGOOptimizeSpheresToVBONonIndexed(I->primitiveCGO, 0, true);
           } else {
-            ok_assert(1, I->shaderCGO = CGOSimplify(I->primitiveCGO, 0, SettingGet_i(G, I->R.cs->Setting, I->R.obj->Setting, cSetting_nb_spheres_quality)));
-            ok_assert(1, CGOCombineBeginEnd(&I->shaderCGO));
+            ok_assert(1, I->shaderCGO = CGOSimplify(I->primitiveCGO, 0, SettingGet_i(G, I->cs->Setting.get(), I->obj->Setting.get(), cSetting_nb_spheres_quality)));
             ok_assert(1, CGOOptimizeToVBONotIndexed(&I->shaderCGO));
           }
           I->shaderCGO->use_shader = true;
         }
-        CGORenderGL(I->shaderCGO, NULL, I->R.cs->Setting, I->R.obj->Setting, info, &I->R);
+        CGORender(I->shaderCGO, NULL, I->cs->Setting.get(), I->obj->Setting.get(), info, I);
       } else {
-        CGORenderGL(I->primitiveCGO, NULL, I->R.cs->Setting, I->R.obj->Setting, info, &I->R);
+        CGORender(I->primitiveCGO, NULL, I->cs->Setting.get(), I->obj->Setting.get(), info, I);
       }
     }
   }
   return;
 ok_except1:
   CGOFree(I->shaderCGO);
-  I->R.fInvalidate(&I->R, I->R.cs, cRepInvPurge);
-  I->R.cs->Active[cRepNonbondedSphere] = false;
+  I->invalidate(cRepInvPurge);
+  I->cs->Active[cRepNonbondedSphere] = false;
 }
 
 Rep *RepNonbondedSphereNew(CoordSet * cs, int state)
@@ -114,12 +111,12 @@ Rep *RepNonbondedSphereNew(CoordSet * cs, int state)
 
   unsigned char *active = NULL;
   int nSphere = 0;
-  float transp =
-    1.f - SettingGet_f(G, cs->Setting, obj->Setting, cSetting_nonbonded_transparency);
-  int ok = true;
 
-  OOAlloc(G, RepNonbondedSphere);
-  CHECKOK(ok, I);
+  float prev_transp = -1;
+  float const transp = SettingGet<float>(G, cs->Setting.get(),
+      obj->Setting.get(), cSetting_nonbonded_transparency);
+
+  int ok = true;
 
   if (ok)
     active = pymol::malloc<unsigned char>(cs->NIndex);
@@ -134,28 +131,19 @@ Rep *RepNonbondedSphereNew(CoordSet * cs, int state)
     }
   }
   if(!nSphere) {
-    OOFreeP(I);
     FreeP(active);
     return (NULL);
   }
   float nb_spheres_size =
-    SettingGet_f(G, cs->Setting, obj->Setting, cSetting_nb_spheres_size);
+    SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_nb_spheres_size);
 
-  RepInit(G, &I->R);
-  I->R.fRender = (void (*)(struct Rep *, RenderInfo *)) RepNonbondedSphereRender;
-  I->R.fFree = (void (*)(struct Rep *)) RepNonbondedSphereFree;
-  I->R.fRecolor = NULL;
-  I->R.obj = (CObject *) (cs->Obj);
-  I->R.cs = cs;
+  auto I = new RepNonbondedSphere(cs, state);
   I->shaderCGO = NULL;
   I->primitiveCGO = NULL;
 
   /* Generate primitiveCGO */
-  float at_transp;
   int NP = 0;
-  bool alpha_set = false;
   I->primitiveCGO = CGONew(G);
-  CGOAlpha(I->primitiveCGO, transp);
   for(int a = 0; ok && a < cs->NIndex; a++){
     if(active[a]) {
       int a1 = cs->IdxToAtm[a];
@@ -172,14 +160,16 @@ Rep *RepNonbondedSphereNew(CoordSet * cs, int state)
         vc = ColorGet(G, c1);
       }
       CGOPickColor(I->primitiveCGO, a1, (ai->masked ? cPickableNoPick : cPickableAtom));
-      if(AtomSettingGetIfDefined(G, ai, cSetting_nonbonded_transparency, &at_transp)){
+
+      auto const at_transp =
+          AtomSettingGetWD(G, ai, cSetting_nonbonded_transparency, transp);
+
+      if (prev_transp != at_transp) {
         CGOAlpha(I->primitiveCGO, 1.f - at_transp);
-        alpha_set = true;
-      } else if (alpha_set){
-        /* if atom level transparency is set, and this atom doesn't have transparency,
-           then set back to object-level transparency */
-        CGOAlpha(I->primitiveCGO, transp);
-        alpha_set = false;
+
+        if (at_transp > 0) {
+          I->setHasTransparency();
+        }
       }
       CGOColorv(I->primitiveCGO, vc);
       CGOSphere(I->primitiveCGO, v1, nb_spheres_size);
@@ -187,14 +177,10 @@ Rep *RepNonbondedSphereNew(CoordSet * cs, int state)
     ok &= !G->Interrupt;
   }
   CGOStop(I->primitiveCGO);
-  I->primitiveCGO->sphere_quality = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_nb_spheres_quality);
-  if (ok){
-    I->R.context.object = obj;
-    I->R.context.state = state;
-  }
+  I->primitiveCGO->sphere_quality = SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_nb_spheres_quality);
   FreeP(active);
   if (!ok){
-    RepNonbondedSphereFree(I);
+    delete I;
     I = NULL;
   }
   return (Rep *) I;
