@@ -19,6 +19,7 @@
 #include "Err.h"
 #include "SpecRec.h"
 #include "AtomIterators.h"
+#include "Feedback.h"
 
 #include <algorithm>
 #include <map>
@@ -34,7 +35,8 @@ enum {
   STORE_ACTIVE = (1 << 1),
   STORE_COLOR  = (1 << 2),
   STORE_REP    = (1 << 3),
-  STORE_FRAME  = (1 << 4)
+  STORE_FRAME  = (1 << 4),
+  STORE_THUMBNAIL = (1 << 5)
 };
 
 void SceneSetNames(PyMOLGlobals * G, const std::vector<std::string> &list);
@@ -46,6 +48,23 @@ const std::vector<std::string> & MovieSceneGetOrder(PyMOLGlobals * G) {
   return G->scenes->order;
 }
 
+/**
+ * Get thumbnail of a scene based on name
+ */
+png_outbuf_t MovieSceneGetThumbnail(PyMOLGlobals* G, pymol::zstring_view name)
+{
+  auto scenes = G->scenes + cMovieSceneStackDefault;
+  auto it = scenes->dict.find(name.c_str());
+
+  if (it == scenes->dict.end()) {
+    return {};
+  }
+
+  png_outbuf_t out_buf = it->second.thumbnail.getVecData();
+
+  return out_buf;
+}
+
 /**
  * Get a unique scene key
  */
@@ -158,6 +177,7 @@ pymol::Result<> MovieSceneOrder(PyMOLGlobals* G, std::vector<std::string> names_
  * @param store_active    store enabled/disabled
  * @param store_rep       store reps
  * @param store_frame     store movie frame
+ * @param store_thumbnail store the thumbnail
  */
 pymol::Result<> MovieSceneStore(PyMOLGlobals * G, const char * name,
     const char * message,
@@ -166,8 +186,10 @@ pymol::Result<> MovieSceneStore(PyMOLGlobals * G, const char * name,
     bool store_active,
     bool store_rep,
     bool store_frame,
+    bool store_thumbnail,
     const char * sele,
-    size_t stack)
+    size_t stack,
+    bool quiet)
 {
   const bool is_defaultstack = stack == cMovieSceneStackDefault;
   auto scenes = G->scenes + stack;
@@ -196,7 +218,8 @@ pymol::Result<> MovieSceneStore(PyMOLGlobals * G, const char * name,
       (store_active ? STORE_ACTIVE : 0) |
       (store_color ? STORE_COLOR : 0) |
       (store_rep ? STORE_REP : 0) |
-      (store_frame ? STORE_FRAME : 0));
+      (store_frame ? STORE_FRAME : 0) |
+      (store_thumbnail ? STORE_THUMBNAIL : 0));
 
   // message
   scene.message = message ? message : "";
@@ -207,6 +230,17 @@ pymol::Result<> MovieSceneStore(PyMOLGlobals * G, const char * name,
   // frame
   scene.frame = SceneGetFrame(G);
 
+  // thumbnail
+  if (store_thumbnail) {
+    int thumbnail_width = 200;
+    int thumbnail_height = 100;
+    scene.thumbnail = pymol::Image(thumbnail_width, thumbnail_height);
+    SceneUpdate(G, false); // Make sure scene is on screen
+    SceneDeferImage(G, scene.thumbnail.getWidth(), scene.thumbnail.getHeight(),
+        nullptr, 0, -1, 0, 1, &scene.thumbnail);
+    SceneInvalidate(G); // Used to refresh the screen
+  }
+
   // atomdata
   if (store_color || store_rep) {
 
@@ -239,7 +273,7 @@ pymol::Result<> MovieSceneStore(PyMOLGlobals * G, const char * name,
     SET_BIT_TO(sceneobj.visRep, 0, rec->visible);
   }
 
-  if (is_defaultstack) {
+  if (is_defaultstack && !quiet) {
     PRINTFB(G, FB_Scene, FB_Details)
     " scene: scene stored as \"%s\".\n", key.c_str() ENDFB(G);
   }
@@ -263,6 +297,41 @@ static void MovieSceneRecallMessage(PyMOLGlobals * G, const std::string &message
 #endif
 }
 
+/**
+ * Get message for a scene based on name
+ * @param name name of the scene
+ */
+std::string MovieSceneGetMessage(PyMOLGlobals* G, pymol::zstring_view name)
+{
+  auto scenes = G->scenes + cMovieSceneStackDefault;
+  auto it = scenes->dict.find(name.c_str());
+
+  if (it == scenes->dict.end()) {
+    return {};
+  }
+
+  return it->second.message;
+}
+
+/**
+ * Set message for a scene based on name
+ * @param name name of the scene
+ * @param message scene's message
+ */
+pymol::Result<> MovieSceneSetMessage(
+    PyMOLGlobals* G, pymol::zstring_view name, pymol::zstring_view message)
+{
+  auto scenes = G->scenes + cMovieSceneStackDefault;
+  auto it = scenes->dict.find(name.c_str());
+
+  if (it == scenes->dict.end()) {
+    return pymol::make_error(name, " could not be found.");
+  } else {
+    it->second.message = message.c_str();
+    return {};
+  }
+}
+
 /**
  * Set the frame or state, depending on whether a movie is defined and/or
  * playing, and depending on the scene_frame_mode setting.
@@ -620,9 +689,9 @@ pymol::Result<> MovieSceneFunc(PyMOLGlobals* G, const MovieSceneFuncArgs& args)
 
   PRINTFB(G, FB_Scene, FB_Blather)
     " MovieScene: key=%s action=%s message=%s store_view=%d store_color=%d"
-    " store_active=%d store_rep=%d animate=%f new_key=%s hand=%d\n",
-    key.c_str(), action.c_str(), args.message.c_str(), args.store_view, args.store_color, args.store_active, args.store_rep,
-    args.animate, args.new_key.c_str(), args.hand
+    " store_active=%d store_rep=%d store_thumbnail=%d animate=%f new_key=%s hand=%d\n",
+    key.c_str(), action.c_str(), args.message.c_str(), args.store_view, args.store_color, args.store_active,
+    args.store_rep, args.store_thumbnail, args.animate, args.new_key.c_str(), args.hand
     ENDFB(G);
 
   // insert_before, insert_after
@@ -657,13 +726,15 @@ pymol::Result<> MovieSceneFunc(PyMOLGlobals* G, const MovieSceneFuncArgs& args)
       ExecutiveSetObjVisib(G, "*", false, false);
       MovieSceneRecallMessage(G, "");
     } else {
-      status = MovieSceneRecall(G, key.c_str(), args.animate, args.store_view, args.store_color,
-          args.store_active, args.store_rep, args.store_frame, args.sele.c_str(), args.stack);
+      status = MovieSceneRecall(G, key.c_str(), args.animate, args.store_view,
+      args.store_color, args.store_active, args.store_rep, args.store_frame,
+      args.sele.c_str(), args.stack);
     }
 
   } else if (action == "store") {
-    status = MovieSceneStore(G, key.c_str(), args.message.c_str(), args.store_view, args.store_color,
-        args.store_active, args.store_rep, args.store_frame, args.sele.c_str(), args.stack);
+    status = MovieSceneStore(G, key.c_str(), args.message.c_str(), args.store_view,
+    args.store_color, args.store_active, args.store_rep, args.store_frame, args.store_thumbnail,
+    args.sele.c_str(), args.stack, args.quiet);
 
     // insert_before, insert_after
     if (status && beforeafter)
@@ -751,14 +822,10 @@ static bool PConvFromPyObject(PyMOLGlobals * G, PyObject * obj, MovieScene &out)
     return false;
   }
 
-  if (!PConvArgsFromPyList(nullptr, obj,
-        out.storemask,
-        out.frame,
-        out.message,
-        out.view,
-        atomdata_old_ids,
-        out.objectdata))
-    /* ignore */;
+  if (!PConvArgsFromPyList(nullptr, obj, out.storemask, out.frame, out.message,
+          out.view, atomdata_old_ids, out.objectdata)) {
+    /* ignore */
+  }
 
   // restore atomdata dict but with converted ids
   PConvFromPyObject(G, PyList_GetItem(obj, 4), atomdata_old_ids);
