@@ -20,7 +20,7 @@ Z* -------------------------------------------------------------------
 #include"os_gl.h"
 
 
-#include "OOMac.h"
+#include "Err.h"
 
 #include "main.h"
 #include "Base.h"
@@ -34,6 +34,7 @@ Z* -------------------------------------------------------------------
 #include "Text.h"
 #include "PyMOL.h"
 #include "CGO.h"
+#include "Block.h"
 
 #define cPopUpLineHeight DIP2PIXEL(17)
 #define cPopUpTitleHeight DIP2PIXEL(19)
@@ -874,7 +875,7 @@ void CPopUp::draw(CGO* orthoCGO)
           }
 
           TextSetPos2i(G, xx, y + cPopUpCharLift);
-          TextDrawChar(G, *(c++) ORTHOCGOARGVAR);
+          TextDrawChar(G, *(c++), orthoCGO);
           xx = xx + DIP2PIXEL(8);
         }
 
