@@ -33,6 +33,7 @@ from . import creating
 from .creating import \
       copy,               \
       create,             \
+      curve_new,          \
       extract,            \
       fragment,           \
       group,              \
@@ -204,6 +205,7 @@ from .editing import \
       invert,             \
       iterate,            \
       iterate_state,      \
+      look_at,            \
       map_set,            \
       map_set_border,     \
       map_double,         \
@@ -211,6 +213,7 @@ from .editing import \
       map_trim,           \
       matrix_copy,        \
       matrix_reset,       \
+      move_on_curve,      \
       mse2met,            \
       pbc_unwrap, \
       pbc_wrap, \
@@ -231,6 +234,7 @@ from .editing import \
       set_dihedral,       \
       set_name,           \
       set_geometry,       \
+      set_discrete,       \
       set_object_color,   \
       set_object_ttt,     \
       set_state_order,    \
@@ -255,7 +259,8 @@ from .editing import \
       vdw_fit
 
 from .editor import \
-      fab
+      fab,                \
+      fnab
 
 from .computing import \
       clean
@@ -362,10 +367,14 @@ from .viewing import \
       enable,             \
       full_screen,        \
       get_colorection,    \
+      get_clip,           \
       get_view,           \
       get_viewport,       \
       get_vis,            \
       get_scene_list,     \
+      get_scene_thumbnail,\
+      get_scene_message,  \
+      set_scene_message,  \
       hide,               \
       ipython_image,      \
       label,              \
